@@ -1,4 +1,4 @@
-# Copyright 2002 Ben Escoto
+# Copyright 2002, 2003 Ben Escoto
 #
 # This file is part of rdiff-backup.
 #
@@ -22,9 +22,14 @@
 from __future__ import generators
 import tempfile, os, cStringIO
 import Globals, Time, Rdiff, Hardlink, rorpiter, selection, rpath, \
-	   log, static, robust, metadata, statistics, TempFile
+	   log, static, robust, metadata, statistics, TempFile, eas_acls
 
 
+# This should be set to selection.Select objects over the source and
+# mirror directories respectively.
+_select_source = None
+_select_mirror = None
+
 # This will be set to the time of the current mirror
 _mirror_time = None
 # This will be set to the exact time to restore to (not restore_to_time)
@@ -43,7 +48,6 @@ def Restore(mirror_rp, inc_rpath, target, restore_to_time):
 	target_iter = TargetS.get_initial_iter(target)
 	diff_iter = MirrorS.get_diffs(target_iter)
 	TargetS.patch(target, diff_iter)
-	MirrorS.close_rf_cache()
 
 def get_inclist(inc_rpath):
 	"""Returns increments with given base"""
@@ -60,20 +64,13 @@ def get_inclist(inc_rpath):
 	return inc_list
 
 def ListChangedSince(mirror_rp, inc_rp, restore_to_time):
-	"""List the changed files under mirror_rp since rest time
+	"""List the changed files under mirror_rp since rest time"""
+	MirrorS = mirror_rp.conn.restore.MirrorStruct
+	MirrorS.set_mirror_and_rest_times(restore_to_time)
+	MirrorS.initialize_rf_cache(mirror_rp, inc_rp)
 
-	Notice the output is an iterator of RORPs.  We do this because we
-	want to give the remote connection the data in buffered
-	increments, and this is done automatically for rorp iterators.
-	Encode the lines in the first element of the rorp's index.
-
-	"""
-	assert mirror_rp.conn is Globals.local_connection, "Run locally only"
-	MirrorStruct.set_mirror_and_rest_times(restore_to_time)
-	MirrorStruct.initialize_rf_cache(mirror_rp, inc_rp)
-
-	old_iter = MirrorStruct.get_mirror_rorp_iter(_rest_time, 1)
-	cur_iter = MirrorStruct.get_mirror_rorp_iter(_mirror_time, 1)
+	cur_iter = MirrorS.get_mirror_rorp_iter(_mirror_time, 1)
+	old_iter = MirrorS.get_mirror_rorp_iter(_rest_time, 1)
 	collated = rorpiter.Collate2Iters(old_iter, cur_iter)
 	for old_rorp, cur_rorp in collated:
 		if not old_rorp: change = "new"
@@ -82,17 +79,17 @@ def ListChangedSince(mirror_rp, inc_rp, restore_to_time):
 		else: change = "changed"
 		path_desc = (old_rorp and old_rorp.get_indexpath() or
 					 cur_rorp.get_indexpath())
-		yield rpath.RORPath(("%-7s %s" % (change, path_desc),))
-	MirrorStruct.close_rf_cache()
+		print "%-7s %s" % (change, path_desc)
+
 
 def ListAtTime(mirror_rp, inc_rp, time):
 	"""List the files in archive at the given time"""
-	assert mirror_rp.conn is Globals.local_connection, "Run locally only"
-	MirrorStruct.set_mirror_and_rest_times(time)
-	MirrorStruct.initialize_rf_cache(mirror_rp, inc_rp)
+	MirrorS = mirror_rp.conn.restore.MirrorStruct
+	MirrorS.set_mirror_and_rest_times(time)
+	MirrorS.initialize_rf_cache(mirror_rp, inc_rp)
 
-	old_iter = MirrorStruct.get_mirror_rorp_iter(_rest_time, 1)
-	for rorp in old_iter: yield rorp
+	old_iter = MirrorS.get_mirror_rorp_iter(_rest_time, 1)
+	for rorp in old_iter: print rorp.get_indexpath()
 	
 
 class MirrorStruct:
@@ -123,34 +120,21 @@ class MirrorStruct:
 		and what was actually on the mirror side will correspond to the
 		older one.
 
-		So if restore_to_time is inbetween two increments, return the
-		older one.
+		So here we assume all rdiff-backup events were recorded in
+		"increments" increments, and if it's in-between we pick the
+		older one here.
 
 		"""
-		inctimes = cls.get_increment_times()
+		global _rest_time
+		base_incs = get_inclist(Globals.rbdir.append("increments"))
+		if not base_incs: return _mirror_time
+		inctimes = [inc.getinctime() for inc in base_incs]
+		inctimes.append(_mirror_time)
 		older_times = filter(lambda time: time <= restore_to_time, inctimes)
 		if older_times: return max(older_times)
 		else: # restore time older than oldest increment, just return that
 			return min(inctimes)
 
-	def get_increment_times(cls, rp = None):
-		"""Return list of times of backups, including current mirror
-
-		Take the total list of times from the increments.<time>.dir
-		file and the mirror_metadata file.  Sorted ascending.
-
-		"""
-		# use dictionary to remove dups
-		if not _mirror_time: d = {cls.get_mirror_time(): None}
-		else: d = {_mirror_time: None}
-		if not rp or not rp.index: rp = Globals.rbdir.append("increments")
-		for inc in get_inclist(rp): d[inc.getinctime()] = None
-		for inc in get_inclist(Globals.rbdir.append("mirror_metadata")):
-			d[inc.getinctime()] = None
-		return_list = d.keys()
-		return_list.sort()
-		return return_list
-
 	def initialize_rf_cache(cls, mirror_base, inc_base):
 		"""Set cls.rf_cache to CachedRF object"""
 		inc_list = get_inclist(inc_base)
@@ -159,10 +143,6 @@ class MirrorStruct:
 		cls.root_rf = rf
 		cls.rf_cache = CachedRF(rf)
 
-	def close_rf_cache(cls):
-		"""Run anything remaining on CachedRF object"""
-		cls.rf_cache.close()
-
 	def get_mirror_rorp_iter(cls, rest_time = None, require_metadata = None):
 		"""Return iter of mirror rps at given restore time
 
@@ -174,11 +154,13 @@ class MirrorStruct:
 
 		"""
 		if rest_time is None: rest_time = _rest_time
-		metadata_iter = metadata.GetMetadata_at_time(Globals.rbdir,
-				 rest_time, restrict_index = cls.mirror_base.index)
-		if metadata_iter: rorp_iter = metadata_iter
-		elif require_metadata: log.Log.FatalError("Mirror metadata not found")
-		else:
+
+		rorp_iter = eas_acls.GetCombinedMetadataIter(
+			Globals.rbdir, rest_time, restrict_index = cls.mirror_base.index,
+			acls = Globals.write_acls, eas = Globals.write_eas)
+		if not rorp_iter:
+			if require_metadata:
+				log.Log.FatalError("Mirror metadata not found")
 			log.Log("Warning: Mirror metadata not found, "
 					"reading from directory", 2)
 			rorp_iter = cls.get_rorp_iter_from_rf(cls.root_rf)
@@ -198,9 +180,7 @@ class MirrorStruct:
 		rorp = rf.get_attribs()
 		yield rorp
 		if rorp.isdir():
-			for sub_rf in rf.yield_sub_rfs():
-				for attribs in cls.get_rorp_iter_from_rf(sub_rf):
-					yield attribs
+			for sub_rf in rf.yield_sub_rfs(): yield sub_rf.get_attribs()
 
 	def subtract_indicies(cls, index, rorp_iter):
 		"""Subtract index from index of each rorp in rorp_iter
@@ -259,16 +239,9 @@ static.MakeClass(MirrorStruct)
 
 class TargetStruct:
 	"""Hold functions to be run on the target side when restoring"""
-	_select = None
-	def set_target_select(cls, target, select_opts, *filelists):
-		"""Return a selection object iterating the rorpaths in target"""
-		cls._select = selection.Select(target)
-		cls._select.ParseArgs(select_opts, filelists)
-		cls._select.set_iter()
-
 	def get_initial_iter(cls, target):
-		"""Return selector previously set with set_initial_iter"""
-		return cls._select or selection.Select(target).set_iter()
+		"""Return a selection object iterating the rorpaths in target"""
+		return selection.Select(target).set_iter()
 
 	def patch(cls, target, diff_iter):
 		"""Patch target with the diffs from the mirror side
@@ -307,8 +280,6 @@ class CachedRF:
 		"""Initialize CachedRF, self.rf_list variable"""
 		self.root_rf = root_rf
 		self.rf_list = [] # list should filled in index order
-		if Globals.process_uid != 0:
-			self.perm_changer = PermissionChanger(root_rf.mirror_rp)
 
 	def list_rfs_in_cache(self, index):
 		"""Used for debugging, return indicies of cache rfs for printing"""
@@ -318,54 +289,28 @@ class CachedRF:
 		return "\n".join((s1, s2, s3))
 
 	def get_rf(self, index):
-		"""Return RestoreFile of given index, or None"""
+		"""Return RestoreFile of given index"""
 		while 1:
-			if not self.rf_list:
-				if not self.add_rfs(index): return None
-			rf = self.rf_list[0]
-			if rf.index == index:
-				if Globals.process_uid != 0: self.perm_changer(index)
-				return rf
-			elif rf.index > index:
-				# Try to add earlier indicies.  But if first is
-				# already from same directory, or we can't find any
-				# from that directory, then we know it can't be added.
-				if (index[:-1] == rf.index[:-1] or not
-					self.add_rfs(index)): return None
-			else: del self.rf_list[0]
+			if not self.rf_list: self.add_rfs(index)
+			rf = self.rf_list.pop(0)
+			if rf.index < index: continue
+			elif rf.index == index: return rf
+			self.rf_list.insert(0, rf)
+			self.add_rfs(index)
 
 	def get_fp(self, index):
 		"""Return the file object (for reading) of given index"""
-		rf = self.get_rf(index)
-		if not rf:
-			log.Log("""Error: Unable to retrieve data for file %s!
-The cause is probably data loss from the destination directory.""" %
-					(index and "/".join(index) or '.',), 2)
-			return cStringIO.StringIO('')
 		return self.get_rf(index).get_restore_fp()
 
 	def add_rfs(self, index):
-		"""Given index, add the rfs in that same directory
-
-		Returns false if no rfs are available, which usually indicates
-		an error.
-
-		"""
+		"""Given index, add the rfs in that same directory"""
 		if not index: return self.root_rf
 		parent_index = index[:-1]
-		if Globals.process_uid != 0: self.perm_changer(parent_index)
 		temp_rf = RestoreFile(self.root_rf.mirror_rp.new_index(parent_index),
 							  self.root_rf.inc_rp.new_index(parent_index), [])
 		new_rfs = list(temp_rf.yield_sub_rfs())
-		if not new_rfs:
-			log.Log("Warning: No RFs added for index %s" % (index,), 2)
-			return 0
+		assert new_rfs, "No RFs added for index %s" % index
 		self.rf_list[0:0] = new_rfs
-		return 1
-
-	def close(self):
-		"""Finish remaining rps in PermissionChanger"""
-		if Globals.process_uid != 0: self.perm_changer.finish()
 
 
 class RestoreFile:
@@ -489,13 +434,7 @@ rdiff-backup destination directory, or a bug in rdiff-backup""" %
 
 	def yield_sub_rfs(self):
 		"""Return RestoreFiles under current RestoreFile (which is dir)"""
-		if not self.mirror_rp.isdir() and not self.inc_rp.isdir():
-			log.Log("""Warning: directory %s seems to be missing from backup!
-
-This is probably due to files being deleted manually from the
-rdiff-backup destination directory.  In general you shouldn't do this,
-as data loss may result.\n""" % (self.mirror_rp.get_indexpath(),), 2)
-			return
+		assert self.mirror_rp.isdir() or self.inc_rp.isdir()
 		if self.mirror_rp.isdir():
 			mirror_iter = self.yield_mirrorrps(self.mirror_rp)
 		else: mirror_iter = iter([])
@@ -510,7 +449,7 @@ as data loss may result.\n""" % (self.mirror_rp.get_indexpath(),), 2)
 				inc_list = []
 			else: inc_rp, inc_list = inc_pair
 			if not mirror_rp:
-				mirror_rp = self.mirror_rp.new_index_empty(inc_rp.index)
+				mirror_rp = self.mirror_rp.new_index(inc_rp.index)
 			yield self.__class__(mirror_rp, inc_rp, inc_list)
 
 	def yield_mirrorrps(self, mirrorrp):
@@ -528,39 +467,29 @@ as data loss may result.\n""" % (self.mirror_rp.get_indexpath(),), 2)
 
 		"""
 		if not inc_rpath.isdir(): return
+		inc_dict = {} # dictionary of basenames:IndexedTuples(index, inc_list)
+		dirlist = robust.listrp(inc_rpath)
 
-		def get_inc_pairs():
-			"""Return unsorted list of (basename, inc_filenames) pairs"""
-			inc_dict = {} # dictionary of basenames:inc_filenames
-			dirlist = robust.listrp(inc_rpath)
+		def affirm_dict_indexed(basename):
+			"""Make sure the rid dictionary has given basename as key"""
+			if not inc_dict.has_key(basename):
+				sub_inc_rp = inc_rpath.append(basename)
+				inc_dict[basename] = rorpiter.IndexedTuple(sub_inc_rp.index,
+														   (sub_inc_rp, []))
 
-			def add_to_dict(filename):
-				"""Add filename to the inc tuple dictionary"""
-				rp = inc_rpath.append(filename)
-				if rp.isincfile() and rp.getinctype() != 'data':
-					basename = rp.getincbase_str()
-					inc_filename_list = inc_dict.setdefault(basename, [])
-					inc_filename_list.append(filename)
-				elif rp.isdir(): inc_dict.setdefault(filename, [])
+		def add_to_dict(filename):
+			"""Add filename to the inc tuple dictionary"""
+			rp = inc_rpath.append(filename)
+			if rp.isincfile() and rp.getinctype() != 'data':
+				basename = rp.getincbase_str()
+				affirm_dict_indexed(basename)
+				inc_dict[basename][1].append(rp)
+			elif rp.isdir(): affirm_dict_indexed(filename)
 
-			for filename in dirlist: add_to_dict(filename)
-			return inc_dict.items()
-
-		def inc_filenames2incrps(filenames):
-			"""Map list of filenames into increment rps"""
-			l = []
-			for filename in filenames:
-				rp = inc_rpath.append(filename)
-				assert rp.isincfile(), rp.path
-				l.append(rp)
-			return l
-
-		items = get_inc_pairs()
-		items.sort() # Sorting on basis of basename now
-		for (basename, inc_filenames) in items:
-			sub_inc_rpath = inc_rpath.append(basename)
-			yield rorpiter.IndexedTuple(sub_inc_rpath.index,
-					  (sub_inc_rpath, inc_filenames2incrps(inc_filenames)))
+		for filename in dirlist: add_to_dict(filename)
+		keys = inc_dict.keys()
+		keys.sort()
+		for key in keys: yield inc_dict[key]
 
 
 class PatchITRB(rorpiter.ITRBranch):
@@ -628,7 +557,7 @@ class PatchITRB(rorpiter.ITRBranch):
 		assert diff_rorp.get_attached_filetype() == 'snapshot'
 		self.dir_replacement = TempFile.new(base_rp)
 		rpath.copy_with_attribs(diff_rorp, self.dir_replacement)
-		if base_rp.isdir(): base_rp.chmod(0700)
+		if base_rp.isdir() and Globals.change_permissions: base_rp.chmod(0700)
 
 	def prepare_dir(self, diff_rorp, base_rp):
 		"""Prepare base_rp to turn into a directory"""
@@ -636,7 +565,7 @@ class PatchITRB(rorpiter.ITRBranch):
 		if not base_rp.isdir():
 			if base_rp.lstat(): base_rp.delete()
 			base_rp.mkdir()
-		base_rp.chmod(0700)
+		if Globals.change_permissions: base_rp.chmod(0700)
 
 	def end_process(self):
 		"""Finish processing directory"""
@@ -648,66 +577,3 @@ class PatchITRB(rorpiter.ITRBranch):
 			self.base_rp.rmdir()
 			if self.dir_replacement.lstat():
 				rpath.rename(self.dir_replacement, self.base_rp)
-
-
-class PermissionChanger:
-	"""Change the permission of mirror files and directories
-
-	The problem is that mirror files and directories may need their
-	permissions changed in order to be read and listed, and then
-	changed back when we are done.  This class hooks into the CachedRF
-	object to know when an rp is needed.
-
-	"""
-	def __init__(self, root_rp):
-		self.root_rp = root_rp
-		self.current_index = ()
-		# Below is a list of (index, rp, old_perm) triples in reverse
-		# order that need clearing
-		self.open_index_list = []
-
-	def __call__(self, index):
-		"""Given rpath, change permissions up to and including index"""
-		old_index = self.current_index
-		self.current_index = index
-		if not index or index <= old_index: return
-		self.restore_old(index)
-		self.add_new(old_index, index)
-
-	def restore_old(self, index):
-		"""Restore permissions for indicies we are done with"""
-		while self.open_index_list:
-			old_index, old_rp, old_perms = self.open_index_list[0]
-			if index[:len(old_index)] > old_index: old_rp.chmod(old_perms)
-			else: break
-			del self.open_index_list[0]
-
-	def add_new(self, old_index, index):
-		"""Change permissions of directories between old_index and index"""
-		for rp in self.get_new_rp_list(old_index, index):
-			if ((rp.isreg() and not rp.readable()) or
-				(rp.isdir() and not rp.hasfullperms())):
-				old_perms = rp.getperms()
-				self.open_index_list.insert(0, (rp.index, rp, old_perms))
-				if rp.isreg(): rp.chmod(0400 | old_perms)
-				else: rp.chmod(0700 | old_perms)
-
-	def get_new_rp_list(self, old_index, index):
-		"""Return list of new rp's between old_index and index
-
-		Do this lazily so that the permissions on the outer
-		directories are fixed before we need the inner dirs.
-
-		"""
-		for i in range(len(index)-1, -1, -1):
-			if old_index[:i] == index[:i]:
-				common_prefix_len = i
-				break
-		else: assert 0
-
-		for total_len in range(common_prefix_len+1, len(index)+1):
-			yield self.root_rp.new_index(index[:total_len])
-
-	def finish(self):
-		"""Restore any remaining rps"""
-		for index, rp, perms in self.open_index_list: rp.chmod(perms)
